/*
 * This file is part of the Black Magic Debug project.
 *
 * Copyright (C) 2011  Black Sphere Technologies Ltd.
 * Written by Gareth McMullin <gareth@blacksphere.co.nz>
 * Copyright (C) 2018 Kevin Redon <kingkevin@cuvoodoo.info>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* This file implements the platform specific functions for the STM32
 * implementation.
 */

#include "general.h"
#include "cdcacm.h"
#include "usbuart.h"
#include "morse.h"

#include <libopencm3/stm32/f1/rcc.h>
#include <libopencm3/cm3/scb.h>
#include <libopencm3/cm3/scs.h>
#include <libopencm3/cm3/nvic.h>
#include <libopencm3/stm32/exti.h>
#include <libopencm3/stm32/usart.h>
#include <libopencm3/usb/usbd.h>
#include <libopencm3/stm32/dac.h>
#include <libopencm3/stm32/i2c.h>

static void i2c_oled_send(const uint8_t* data, uint32_t length);

/* This doesn't concern the BusVoodoo
 */
int platform_hwversion(void)
{
	return 0;
}

void platform_init(void)
{
	SCS_DEMCR |= SCS_DEMCR_VC_MON_EN;
#ifdef ENABLE_DEBUG
	void initialise_monitor_handles(void);
	initialise_monitor_handles();
#endif

	rcc_clock_setup_in_hse_8mhz_out_72mhz();

	/* Enable peripherals */
	rcc_periph_clock_enable(RCC_USB);
	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_GPIOB);
	rcc_periph_clock_enable(RCC_GPIOC);
	rcc_periph_clock_enable(RCC_AFIO);
	rcc_periph_clock_enable(RCC_CRC);

	/* Setup GPIO ports */
	gpio_set_mode(JTAG_PORT, GPIO_MODE_OUTPUT_50_MHZ,
			GPIO_CNF_OUTPUT_PUSHPULL,
			TMS_PIN | TCK_PIN | TDI_PIN);
	gpio_set_mode(LED_PORT, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_PUSHPULL,
			LED_UART | LED_IDLE_RUN | LED_ERROR);
	gpio_clear(SRST_PORT, SRST_PIN);
	gpio_set_mode(SRST_PORT, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_PUSHPULL,
			SRST_PIN);

	/* Disable JTAG (but keep SWD) to GPIOs (PA15, PB3, PB4). */
	gpio_primary_remap(AFIO_MAPR_SWJ_CFG_JTAG_OFF_SW_ON, 0);

	/* Configure power outputs as open-drain (off when high). */
	gpio_set(PWR_BR_PORT, PWR_BR_PIN);
	gpio_set_mode(PWR_BR_PORT, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_OPENDRAIN, PWR_BR_PIN);

	/* Disable all unused BusVoodoo peripherals:
	 * - embedded pull-ups
	 * - high voltage boost converter
	 * - RS-232 transceiver
	 * - RS-485 transceiver
	 * - CAN transceiver
	 */
	gpio_set(GPIOA, GPIO15);
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_OPENDRAIN, GPIO15);
	gpio_set(GPIOB, GPIO4);
	gpio_set_mode(GPIOB, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_OPENDRAIN, GPIO4);
	gpio_set(GPIOA, GPIO7);
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_OPENDRAIN, GPIO7);
	gpio_set(GPIOB, GPIO5);
	gpio_set_mode(GPIOB, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_OPENDRAIN, GPIO5);
	gpio_clear(GPIOC, GPIO15);
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_PUSHPULL, GPIO15);
	gpio_set(GPIOC, GPIO13);
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_OPENDRAIN, GPIO13);
	gpio_clear(GPIOC, GPIO14);
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_PUSHPULL, GPIO14);
	gpio_set(GPIOC, GPIO7);
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_OPENDRAIN, GPIO7);
	
	/* Configure LV at 3.3V for TRST using DAC. */
	gpio_set_mode(GPIOA, GPIO_MODE_INPUT,
		GPIO_CNF_INPUT_ANALOG, GPIO4);
	rcc_periph_clock_enable(RCC_DAC);
	dac_disable(CHANNEL_1);
	dac_buffer_enable(CHANNEL_1);
	dac_set_trigger_source(DAC_CR_TSEL1_SW);
	dac_load_data_buffer_single(((0.8*(1+30.0/10.0)-3.3)*(10.0/30.0)+0.8)/3.3*4085,
			RIGHT12, CHANNEL_1);
	dac_software_trigger(CHANNEL_1);

	/* Remap TIM3 for SWO on TDO. */
	gpio_primary_remap(AFIO_MAPR_SWJ_CFG_JTAG_OFF_SW_ON,
			AFIO_MAPR_TIM3_REMAP_FULL_REMAP);
	gpio_set_mode(GPIOC, GPIO_MODE_INPUT,
			GPIO_CNF_INPUT_FLOAT, GPIO6);

	/* Relocate interrupt vector table here. */
	extern int vector_table;
	SCB_VTOR = (uint32_t)&vector_table;

	platform_timing_init();
	cdcacm_init();

	/* Don't enable UART if we're being debugged. */
	if (!(SCS_DEMCR & SCS_DEMCR_TRCENA)) {
		usbuart_init();
	}

	/* Configure I2C for OLED display */
	gpio_set(GPIOB, GPIO6);
	gpio_set_mode(GPIOB, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_ALTFN_OPENDRAIN, GPIO6);
	gpio_set(GPIOB, GPIO7);
	gpio_set_mode(GPIOB, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_ALTFN_OPENDRAIN, GPIO7);
	rcc_periph_clock_enable(RCC_I2C1);
	i2c_reset(I2C1);
	i2c_peripheral_disable(I2C1);
	i2c_set_clock_frequency(I2C1, rcc_apb1_frequency/1000000);
	i2c_set_fast_mode(I2C1);
	i2c_set_ccr(I2C1, rcc_apb1_frequency/(400000*2));
	i2c_set_trise(I2C1, (300/(1000/(rcc_apb1_frequency/1000000)))+1);
	i2c_peripheral_enable(I2C1);

	/* Display pinout on OLED screen. */
	for(uint32_t i = 0; i < 1000000; i++) { // wait for the OLED to boot
		asm("nop");
	}
	const uint8_t oled_init[] = {
		0x00, // control byte: continuous (multiple byes), command
		0xae, // Set Display ON/OFF: OFF
		// hardware configuration
		0xa8, 0x3f, // Set Multiplex Ratio: 64
		0xd3, 0x00, // Set Display Offset: 0
		0xa1, // Set Segment Re-map: column address 0 is mapped to SEG127
		0xc8, // Set COM Output Scan Direction: normal mode (RESET) Scan from COM[N-1] to COM[0]
		0xda, 0x12, // Set COM Pins Hardware Configuration: Alternative COM pin configuration, Disable COM Left/Right remap
		0x40, // Set Display Start Line: start line register from 0
		// fundamental commands
		0x81, 0xff, // Set Contrast Control: 256
		0xa6, // Set Normal/Inverse Display: Normal display (RESET)
		// Timing & Driving Scheme Setting
		0xd5, 0xf0, // Set Display Clock Divide Ratio/Oscillator Frequency: Divide ratio=129, F_OSC=1
		0xd9, 0x22, // Set Pre-charge Period: Phase 1=2 DCLK, Phase 2=2DCLK
		0xdb, 0x20, // Set V_COMH Deselect Level: ~0.77xV_CC
		// Charge Pump
		0x8d, 0x14, // Charge Pump Setting: Enable Charge Pump
		// Addressing Setting
		0x20, 0x00, // Set Memory Addressing Mode: Horizontal Addressing Mode
		0xb0, // Set Page Start Address for Page Addressing Mode: PAGE0
		0x00, // Set Lower Column Start Address for Page Addressing Mode: 0
		0x10 // Set Higher Column Start Address for Page Addressing Mode: 0
	}; // command to initialize the display
	i2c_oled_send(oled_init, sizeof(oled_init)/sizeof(oled_init[0]));
	const uint8_t oled_pinout_display_buffer[] = {
		0x40, // control byte: continuous (multiple byes), data
		0x00, 0xfe, 0xfe, 0x86, 0x86, 0x86, 0x86, 0xce, 0xfc, 0x78, 0x00, 0xfe, 0xfe, 0x1c, 0x38, 0x70, 0x38, 0x1c, 0xfe, 0xfe, 0x00, 0xfe, 0xfe, 0x06, 0x06, 0x06, 0x06, 0x8e, 0xfc, 0xf8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x06, 0x06, 0x06, 0xfe, 0xfe, 0x00, 0x00, 0x00, 0x06, 0x06, 0x06, 0xfe, 0xfe, 0xfe, 0x06, 0x06, 0x06, 0x00, 0xf8, 0xfc, 0x8e, 0x86, 0x86, 0x86, 0x8e, 0xfc, 0xf8, 0x00, 0xfc, 0xfe, 0x0e, 0x06, 0x06, 0x86, 0x86, 0x8e, 0x8c, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe0, 0x78, 0x1e, 0x06, 0x00, 0x00, 0x78, 0xfc, 0xce, 0x86, 0x86, 0x86, 0x86, 0x06, 0x06, 0x00, 0xfe, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xfe, 0x00, 0xfe, 0xfe, 0x06, 0x06, 0x06, 0x0e, 0x1c, 0xf8, 0xf0, 0x00, 0x7f, 0x7f, 0x61, 0x61, 0x61, 0x61, 0x73, 0x3f, 0x1e, 0x00, 0x7f, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x00, 0x7f, 0x7f, 0x03, 0x03, 0x03, 0x03, 0x03, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x70, 0x38, 0x1c, 0x0f, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x7f, 0x00, 0x00, 0x00, 0x00, 0x7f, 0x7f, 0x01, 0x01, 0x01, 0x01, 0x01, 0x7f, 0x7f, 0x00, 0x3f, 0x7f, 0x70, 0x60, 0x60, 0x61, 0x61, 0x7f, 0x3f, 0x00, 0x60, 0x78, 0x1e, 0x07, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x60, 0x61, 0x61, 0x61, 0x61, 0x73, 0x3f, 0x1e, 0x00, 0x1f, 0x3f, 0x78, 0x3c, 0x1e, 0x3c, 0x78, 0x3f, 0x1f, 0x00, 0x7f, 0x7f, 0x60, 0x60, 0x60, 0x70, 0x38, 0x1f, 0x0f, 0xff, 0xff, 0x03, 0x03, 0x13, 0x13, 0xf3, 0x13, 0x13, 0x03, 0xf3, 0x23, 0x43, 0x23, 0xf3, 0x03, 0x63, 0x93, 0x93, 0x93, 0x13, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x13, 0x13, 0xf3, 0x13, 0x13, 0x03, 0xf3, 0x13, 0x13, 0x13, 0xe3, 0x03, 0xe3, 0x13, 0x13, 0x13, 0xe3, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x13, 0x93, 0x93, 0x63, 0x03, 0x73, 0x83, 0x03, 0x83, 0x73, 0x03, 0x03, 0x13, 0x93, 0x93, 0x63, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xc0, 0x40, 0x47, 0x40, 0x80, 0x00, 0x07, 0x40, 0xc0, 0x40, 0x07, 0x00, 0x84, 0x44, 0x44, 0x44, 0x83, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x40, 0x47, 0x40, 0x40, 0x00, 0xc7, 0x04, 0x04, 0x04, 0xc3, 0x00, 0x83, 0x44, 0x44, 0x44, 0x83, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0x12, 0x32, 0x52, 0x8c, 0x00, 0xc6, 0x28, 0x10, 0x28, 0xc6, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x04, 0x04, 0x04, 0x03, 0x00, 0xc0, 0x41, 0x46, 0x41, 0x80, 0x00, 0xc0, 0x04, 0x04, 0x04, 0xc3, 0x00, 0xc0, 0x40, 0x40, 0x40, 0x80, 0x00, 0xfe, 0x01, 0x01, 0x01, 0x11, 0xf1, 0x00, 0xff, 0x04, 0x08, 0x10, 0x20, 0xff, 0x00, 0xff, 0x01, 0x01, 0x01, 0x01, 0xfe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x1f, 0x10, 0x10, 0x10, 0x0f, 0x00, 0x00, 0x10, 0x1f, 0x10, 0x00, 0x00, 0x0f, 0x10, 0x10, 0x10, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x12, 0x12, 0x12, 0x0c, 0x00, 0x07, 0x18, 0x06, 0x18, 0x07, 0x00, 0x0f, 0x10, 0x10, 0x10, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0f, 0x12, 0x12, 0x10, 0x00, 0x00, 0x1f, 0x02, 0x02, 0x02, 0x01, 0x00, 0x07, 0x18, 0x06, 0x18, 0x07, 0x00, 0x1f, 0x02, 0x06, 0x0a, 0x11, 0x00, 0x00, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0x04, 0x04, 0xfc, 0x04, 0x04, 0x00, 0xf8, 0x04, 0x04, 0x04, 0x04, 0x00, 0xfc, 0x20, 0x50, 0x88, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x80, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x00, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80, 0x80, 0x00, 0x80, 0x80, 0x80, 0x80, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x9c, 0x14, 0x14, 0x14, 0xe4, 0x00, 0x1c, 0x60, 0x80, 0x60, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0x00, 0x00, 0xe0, 0x10, 0x11, 0x10, 0x10, 0x00, 0xf0, 0x01, 0x01, 0x01, 0x01, 0x00, 0xf1, 0x80, 0x40, 0x20, 0x11, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x3f, 0x20, 0x20, 0x20, 0x1f, 0x00, 0x00, 0x20, 0x3f, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x31, 0x0a, 0x04, 0x0a, 0x31, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x23, 0x24, 0x24, 0x24, 0x18, 0x00, 0x3f, 0x04, 0x0c, 0x14, 0x23, 0x00, 0x23, 0x24, 0x24, 0x24, 0x18, 0x00, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0xe0, 0x80, 0x80, 0x00, 0x00, 0x00, 0xf0, 0x91, 0x91, 0x91, 0x60, 0x00, 0xf0, 0x00, 0x81, 0x00, 0xf0, 0x00, 0xf0, 0x90, 0x90, 0x90, 0x60, 0x00, 0x00, 0x00, 0xff, 0xff, 0xff, 0xff, 0xc0, 0xc0, 0xc3, 0xc4, 0xc4, 0xc4, 0xc4, 0xc0, 0xc7, 0xc4, 0xc4, 0xc4, 0xc4, 0xc0, 0xc7, 0xc0, 0xc1, 0xc2, 0xc4, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc3, 0xc4, 0xc4, 0xc4, 0xc0, 0xc0, 0xc7, 0xc0, 0xc0, 0xc0, 0xc0, 0xc0, 0xc1, 0xc6, 0xc1, 0xc6, 0xc1, 0xc0, 0xc7, 0xc0, 0xc1, 0xc2, 0xc4, 0xc0, 0xc0, 0xc0, 0xff, 0xff,
	};
	i2c_oled_send(oled_pinout_display_buffer,
			sizeof(oled_pinout_display_buffer)/sizeof(oled_pinout_display_buffer[0]));
	const uint8_t oled_display_on[] = {
		0x80, // control byte: no continuation, command
		0xaf, // Set Display ON/OFF: ON
	}; // command to switch on display
	i2c_oled_send(oled_display_on,
			sizeof(oled_display_on)/sizeof(oled_display_on[0]));
}

bool platform_target_get_power(void)
{
	return !gpio_get(PWR_BR_PORT, PWR_BR_PIN);
}

void platform_target_set_power(bool power)
{
	gpio_set_val(PWR_BR_PORT, PWR_BR_PIN, !power);
}

void platform_srst_set_val(bool assert)
{
	if (assert) {
		gpio_clear(SRST_PORT, SRST_PIN);
		dac_disable(CHANNEL_1);
	} else {
		gpio_set(SRST_PORT, SRST_PIN);
		dac_enable(CHANNEL_1);
	}
		
	gpio_set_val(SRST_PORT, SRST_PIN, !assert);
	if (assert) {
		for(int i = 0; i < 100000; i++) {
			asm("nop");
		}
	}
}

bool platform_srst_get_val()
{
	return gpio_get(SRST_PORT, SRST_PIN)==0;
}

const char *platform_target_voltage(void)
{
	return "unknown";
}

void platform_request_boot(void)
{
	/* Disconnect USB cable by resetting USB Device and pulling USB_DP low*/
	rcc_periph_reset_pulse(RST_USB);
	rcc_periph_clock_enable(RCC_USB);
	rcc_periph_clock_enable(RCC_GPIOA);
	gpio_clear(GPIOA, GPIO12);
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_2_MHZ,
	              GPIO_CNF_OUTPUT_OPENDRAIN, GPIO12);

	/* Enable pull-up on GPIOC4 (connected to DFU pad) to assert
	   Black Magic bootloader */
	GPIOC_CRL = (GPIOC_CRL & 0xfff0ffff) | 0x80000;

	/* Clear reset flags to assert BusVoodoo bootloader */
	RCC_CSR |= RCC_CSR_RMVF; // clear reset flags
}

static void i2c_oled_send(const uint8_t* data, uint32_t length)
{
	i2c_send_start(I2C1);
	for(uint32_t i = 0; i < 100; i++) {
		asm("nop");
	}
	if ((I2C1_SR1 & I2C_SR1_SB) && (I2C1_SR2 & I2C_SR2_MSL)) {
		i2c_send_7bit_address(I2C1, 0x3c, I2C_WRITE);
		for(uint32_t i = 0; i < 500; i++) {
			asm("nop");
		}
		if ((I2C1_SR1 & I2C_SR1_ADDR) && (I2C1_SR2 & I2C_SR2_TRA)) {
			for (uint32_t i=0; i<length; i++) {
				while (!(I2C1_SR1 & I2C_SR1_TxE));
				i2c_send_data(I2C1, data[i]);
			}
			while (!(I2C1_SR1 & I2C_SR1_TxE));
		}
	}
	i2c_send_stop(I2C1);
	while (I2C1_SR2 & I2C_SR2_MSL);
}
