/*
 * This file is part of the Black Magic Debug project.
 *
 * Copyright (C) 2013 Gareth McMullin <gareth@blacksphere.co.nz>
 * Copyright (C) 2018 Kevin Redon <kingkevin@cuvoodoo.info>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <libopencm3/cm3/systick.h>
#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/gpio.h>
#include <libopencm3/cm3/scb.h>

#include "usbdfu.h"
#include "platform.h"

uint32_t app_address = 0x08002000;
int dfu_activity_counter = 0;

void dfu_detach(void)
{
	/* Disconnect USB cable by resetting USB Device and pulling USB_DP low*/
	rcc_periph_reset_pulse(RST_USB);
	rcc_periph_clock_enable(RCC_USB);
	rcc_periph_clock_enable(RCC_GPIOA);
	gpio_clear(GPIOA, GPIO12);
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_2_MHZ, GPIO_CNF_OUTPUT_OPENDRAIN,
	              GPIO12);
	/* USB device must detach, we just reset... */
	scb_reset_system();
}

int main(void)
{
	/* Check if application did not set the DFU pin PC4 to input
	   pull-up/down before reset. */
	if ((GPIOC_CRL & 0x40000) == 0x40000) {
		/* Check if the force DFU bootloader pin PC4 is shorted to 3V3. */
		rcc_periph_clock_enable(RCC_GPIOC);
		gpio_set_mode(GPIOC, GPIO_MODE_INPUT, GPIO_CNF_INPUT_PULL_UPDOWN,
				GPIO4);
		gpio_clear(GPIOC, GPIO4);
		for (uint32_t i=0; i<10; i++) {
			asm("nop");
		}
		if (!gpio_get(GPIOC, GPIO4)) {
			dfu_jump_app_if_valid();
		}
	}

	dfu_protect(DFU_MODE);

	rcc_clock_setup_in_hse_8mhz_out_72mhz();
	systick_set_clocksource(STK_CSR_CLKSOURCE_AHB_DIV8);
	systick_set_reload(900000);

	/* Configure USB related clocks and pins. */
	rcc_periph_clock_enable(RCC_GPIOA);
	rcc_periph_clock_enable(RCC_USB);
	gpio_set_mode(GPIOA, GPIO_MODE_INPUT, 0, GPIO8);

	systick_interrupt_enable();
	systick_counter_enable();

	/* Configure the LED pins. */
	gpio_set_mode(LED_PORT, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_PUSHPULL, LED_PIN);
	gpio_clear(LED_PORT, LED_PIN);

	dfu_init(&st_usbfs_v1_usb_driver, DFU_MODE);

	/* Configure the USB pull-up pin. */
	gpio_set(GPIOA, GPIO8);
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_2_MHZ,
			GPIO_CNF_OUTPUT_PUSHPULL, GPIO8);

	dfu_main();
}

void dfu_event(void)
{
	static bool led_off = false;

	/* Prevent the sys_tick_handler from setting red LED for a bit. */
	dfu_activity_counter = 10;

	/* Toggle the blue activity LED. */
	if (led_off) {
		gpio_set_mode(LED_PORT, GPIO_MODE_INPUT, GPIO_CNF_INPUT_FLOAT,
				LED_PIN);
	} else {
		gpio_set_mode(LED_PORT, GPIO_MODE_OUTPUT_2_MHZ,
				GPIO_CNF_OUTPUT_PUSHPULL, LED_PIN);
		gpio_set(LED_PORT, LED_PIN);
	}
	led_off = !led_off;
}

void sys_tick_handler(void)
{
	/* Set red LED only if there is no DFU activity. */
	if (dfu_activity_counter != 0) {
		dfu_activity_counter--;
	} else {
		gpio_set_mode(LED_PORT, GPIO_MODE_OUTPUT_2_MHZ,
				GPIO_CNF_OUTPUT_PUSHPULL, LED_PIN);
		gpio_clear(LED_PORT, LED_PIN);
	}
}
